@@ -1,4 +1,4 @@
-from security.throttling import PerRequestThrottlingValidator
+from security.throttling.validators import PerRequestThrottlingValidator
 
 
 default_validators = (
